@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# Gufw 15.04.0 - http://gufw.org
+# Gufw 15.10.0 - http://gufw.org
 # Copyright (C) 2008-2015 Marcos Alvarez Costales https://launchpad.net/~costales
 #
 # Gufw is free software; you can redistribute it and/or modify
@@ -16,123 +16,148 @@
 # along with Gufw; if not, see http://www.gnu.org/licenses for more
 # information.
 
-import time, os, commands, ConfigParser
+import time, os, shutil, subprocess, ConfigParser
 
 
 class Backend():
+    UFW_PATH    = '/usr/sbin/ufw'
+    UFW_DEFAULT = '/etc/default/ufw'
+    UFW_CONF    = '/etc/ufw/ufw.conf'
+    UFW_SYSCTL  = '/etc/ufw/sysctl.conf'
+    GUFW_PATH   = '/etc/gufw'
+    GUFW_CFG    = '/etc/gufw/gufw.cfg'
+    GUFW_LOG    = '/var/log/gufw.log'
+    
+    def _run_cmd(self, cmd, lang_c=False):
+        if lang_c:
+            proc = subprocess.Popen(cmd, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env={'LANG':'C'})
+        else:
+            proc = subprocess.Popen(cmd, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        stdout,stderr=proc.communicate()
+        
+        if stderr: # Error
+            while stderr[-1:] == '\n':
+                stderr = stderr[:-1]
+            return stderr
+        else: # OK
+            while stdout[-1:] == '\n':
+                stdout = stdout[:-1]
+            return stdout
+    
     def get_status(self):
-        ufw_status = commands.getstatusoutput('LANGUAGE=C ufw status')
-        if 'Status: active' in ufw_status[1]:
+        if 'Status: active' in self._run_cmd([self.UFW_PATH, 'status'], True):
             return True
         else:
             return False
     
     def get_policy(self, policy):
         if policy == 'incoming':
-            ufw_default_incoming = commands.getstatusoutput('grep DEFAULT_INPUT_POLICY /etc/default/ufw')
-            if 'ACCEPT' in ufw_default_incoming[1]:
+            ufw_default_incoming = self._run_cmd(['grep', 'DEFAULT_INPUT_POLICY', self.UFW_DEFAULT])
+            if 'ACCEPT' in ufw_default_incoming:
                 return 'allow'
-            elif 'DROP' in ufw_default_incoming[1]:
+            elif 'DROP' in ufw_default_incoming:
                 return 'deny'
-            elif 'REJECT' in ufw_default_incoming[1]:
+            elif 'REJECT' in ufw_default_incoming:
                 return 'reject'
         
         elif policy == 'outgoing':
-            ufw_default_outgoing = commands.getstatusoutput('grep DEFAULT_OUTPUT_POLICY /etc/default/ufw')
-            if 'ACCEPT' in ufw_default_outgoing[1]:
+            ufw_default_outgoing = self._run_cmd(['grep', 'DEFAULT_OUTPUT_POLICY', self.UFW_DEFAULT])
+            if 'ACCEPT' in ufw_default_outgoing:
                 return 'allow'
-            elif 'DROP' in ufw_default_outgoing[1]:
+            elif 'DROP' in ufw_default_outgoing:
                 return 'deny'
-            elif 'REJECT' in ufw_default_outgoing[1]:
+            elif 'REJECT' in ufw_default_outgoing:
                 return 'reject'
         
         elif policy == 'routed':
-            ufw_default_outgoing = commands.getstatusoutput('grep "net/ipv4/ip_forward=" /etc/ufw/sysctl.conf')
-            routed_status = ufw_default_outgoing[1].replace(" ", "")
+            ufw_default_outgoing = self._run_cmd(['grep', 'net/ipv4/ip_forward=', self.UFW_SYSCTL])
+            routed_status = ufw_default_outgoing.replace(" ", "")
             if "#net/ipv4/ip_forward=1" in routed_status:
                 return 'disabled'
             
-            ufw_default_outgoing = commands.getstatusoutput('grep DEFAULT_FORWARD_POLICY /etc/default/ufw')
-            if 'ACCEPT' in ufw_default_outgoing[1]:
+            ufw_default_outgoing = self._run_cmd(['grep', 'DEFAULT_FORWARD_POLICY', self.UFW_DEFAULT])
+            if 'ACCEPT' in ufw_default_outgoing:
                 return 'allow'
-            elif 'DROP' in ufw_default_outgoing[1]:
+            elif 'DROP' in ufw_default_outgoing:
                 return 'deny'
-            elif 'REJECT' in ufw_default_outgoing[1]:
+            elif 'REJECT' in ufw_default_outgoing:
                 return 'reject'
     
     def get_ufw_logging(self):
-        ufw_cmd = commands.getstatusoutput('cat /etc/ufw/ufw.conf')
-        if 'LOGLEVEL=full' in ufw_cmd[1]:
+        ufw_cmd = self._run_cmd(['cat', self.UFW_CONF])
+        if 'LOGLEVEL=full' in ufw_cmd:
             return 'full'
-        elif 'LOGLEVEL=high' in ufw_cmd[1]:
+        elif 'LOGLEVEL=high' in ufw_cmd:
             return 'high'
-        elif 'LOGLEVEL=medium' in ufw_cmd[1]:
+        elif 'LOGLEVEL=medium' in ufw_cmd:
             return 'medium'
-        elif 'LOGLEVEL=low' in ufw_cmd[1]:
+        elif 'LOGLEVEL=low' in ufw_cmd:
             return 'low'
         else:
             return 'off'
     
     def set_status(self, status):
-        if status:
-            cmd = 'ufw --force enable'
+        if not status:
+            cmd = [self.UFW_PATH, 'disable']
         else:
-            cmd = 'ufw disable'
+            cmd = [self.UFW_PATH, '--force', 'enable']
         
-        commands.getstatusoutput(cmd)
+        self._run_cmd(cmd)
     
     def set_policy(self, policy, value):
         if policy == 'incoming':
             if value == 'allow':
-                cmd = 'ufw default allow incoming'
+                cmd = [self.UFW_PATH, 'default', 'allow', 'incoming']
             elif value == 'deny':
-                cmd = 'ufw default deny incoming'
+                cmd = [self.UFW_PATH, 'default', 'deny', 'incoming']
             elif value == 'reject':
-                cmd = 'ufw default reject incoming'
+                cmd = [self.UFW_PATH, 'default', 'reject', 'incoming']
         
         elif policy == 'outgoing':
             if value == 'allow':
-                cmd = 'ufw default allow outgoing'
+                cmd = [self.UFW_PATH, 'default', 'allow', 'outgoing']
             elif value == 'deny':
-                cmd = 'ufw default deny outgoing'
+                cmd = [self.UFW_PATH, 'default', 'deny', 'outgoing']
             elif value == 'reject':
-                cmd = 'ufw default reject outgoing'
+                cmd = [self.UFW_PATH, 'default', 'reject', 'outgoing']
         
         elif policy == 'routed':
             if value == 'allow':
-                cmd = 'ufw default allow routed'
+                cmd = [self.UFW_PATH, 'default', 'allow', 'routed']
             elif value == 'deny':
-                cmd = 'ufw default deny routed'
+                cmd = [self.UFW_PATH, 'default', 'deny', 'routed']
             elif value == 'reject':
-                cmd = 'ufw default reject routed'
+                cmd = [self.UFW_PATH, 'default', 'reject', 'routed']
         
-        commands.getstatusoutput(cmd)
+        if cmd:
+            self._run_cmd(cmd)
     
     def set_ufw_logging(self, logging):
         if logging == 'off':
-            cmd = 'ufw logging off'
+            cmd = [self.UFW_PATH, 'logging', 'off']
         elif logging == 'low':
-            cmd = 'ufw logging low'
+            cmd = [self.UFW_PATH, 'logging', 'low']
         elif logging == 'medium':
-            cmd = 'ufw logging medium'
+            cmd = [self.UFW_PATH, 'logging', 'medium']
         elif logging == 'high':
-            cmd = 'ufw logging high'
+            cmd = [self.UFW_PATH, 'logging', 'high']
         elif logging == 'full':
-            cmd = 'ufw logging full'
+            cmd = [self.UFW_PATH, 'logging', 'full']
         
-        commands.getstatusoutput(cmd)
+        if cmd:
+            self._run_cmd(cmd)
     
     def reset_fw(self):
-        cmd = 'ufw --force reset'
-        commands.getstatusoutput(cmd)
+        cmd = [self.UFW_PATH, '--force', 'reset']
+        self._run_cmd(cmd, True)
     
     def get_cfg_value(self, attribute):
         cfg = ConfigParser.ConfigParser()
         try:
-            if not cfg.read(['/etc/gufw/gufw.cfg']):
+            if not cfg.read([self.GUFW_CFG]):
                 return ''
         except:
-            os.remove('/etc/gufw/gufw.cfg')
+            os.remove(self.GUFW_CFG)
             return ''
         
         if not cfg.has_option('GufwConfiguration', attribute):
@@ -141,14 +166,14 @@ class Backend():
         return cfg.get('GufwConfiguration', attribute)
     
     def set_cfg_value(self, attr, value):
-        cfg_file = '/etc/gufw/gufw.cfg'
-        if not os.path.exists(cfg_file):
+        cfg_file = self.GUFW_CFG
+        if not os.path.isfile(cfg_file):
             f = open(cfg_file, 'w')
             cfg = ConfigParser.ConfigParser()
             cfg.add_section("GufwConfiguration")
             cfg.write(f)  
             f.close()  
-            os.chmod(cfg_file, 0700) # Just an admin can read this file
+            os.chmod(cfg_file, 0600) # Just an admin can read this file
         
         cfg = ConfigParser.ConfigParser()
         cfg.read(cfg_file)
@@ -158,8 +183,11 @@ class Backend():
         f.close()
     
     def set_profile_values(self, profile, status, incoming, outgoing, routed, rules):
+        if not os.path.exists(self.GUFW_PATH):
+            os.makedirs(self.GUFW_PATH)
+        
         file_name = profile + '.profile'
-        file_path = os.path.join('/etc', 'gufw', file_name)
+        file_path = os.path.join(self.GUFW_PATH, file_name)
         
         cfg = ConfigParser.ConfigParser()
         
@@ -192,11 +220,11 @@ class Backend():
             cfg.set(section, 'routed',      rule['routed']) 
             cfg.set(section, 'logging',     rule['logging'] )
             i += 1
-            
+        
         f = open(file_path, 'w')
         cfg.write(f)  
         f.close()
-        os.chmod(file_path, 0700) # Just an admin can read this file
+        os.chmod(file_path, 0600) # Just an admin can read this file
     
     def get_profile_values(self, profile):
         status = False
@@ -221,10 +249,10 @@ class Backend():
         for section in cfg.sections():
             
             if section == 'fwBasic':
-                if cfg.get('fwBasic', 'status') == 'enabled':
-                    status = True
-                else:
+                if cfg.get('fwBasic', 'status') == 'disabled':
                     status = False
+                else:
+                    status = True
                 incoming = cfg.get('fwBasic', 'incoming')
                 outgoing = cfg.get('fwBasic', 'outgoing')
                 # Previous Gufw profiles
@@ -258,21 +286,39 @@ class Backend():
     
     
     def delete_file_profile(self, profile):
-        commands.getstatusoutput('rm /etc/gufw/' + profile + '.profile')
+        dst = os.path.join(self.GUFW_PATH, profile + '.profile')
+        try:
+            os.remove(dst)
+        except:
+            pass
     
     def rename_file_profile(self, old, new):
-        commands.getstatusoutput('mv /etc/gufw/' + old + '.profile /etc/gufw/' + new + '.profile')
+        src = os.path.join(self.GUFW_PATH, old + '.profile')
+        dst = os.path.join(self.GUFW_PATH, new + '.profile')
+        
+        if os.path.isfile(dst): # User could import a profile with same English name > It's OK
+            return
+        print('renaming '+src+' a '+dst)
+        try:
+            os.rename(src, dst)
+        except:
+            pass
     
-    def export_profile(self, profile, file):
-        commands.getstatusoutput('cp /etc/gufw/' + profile + '.profile ' + file + ' ; chmod 777 ' + file)
+    def export_profile(self, profile, dst):
+        src = os.path.join(self.GUFW_PATH, profile + '.profile')
+        try:
+            shutil.copyfile(src, dst)
+            os.chmod(dst, 0600) # Avoid other users change a profile
+        except:
+            pass
     
     def refresh_log(self):
-        f = file('/var/log/gufw.log', 'w')
+        f = file(self.GUFW_LOG, 'w')
         f.close()
     
     def add_to_log(self, msg):
         try:
-            f = open('/var/log/gufw.log', 'r')
+            f = open(self.GUFW_LOG, 'r')
             log = f.readlines()
             f.close()
         except:
@@ -282,7 +328,7 @@ class Backend():
         if log:
             new_line = new_line + '\n'
         
-        f = open('/var/log/gufw.log', 'w')
+        f = open(self.GUFW_LOG, 'w')
         f.write(new_line)
         f.writelines(log)
         f.close()
@@ -290,15 +336,15 @@ class Backend():
         return new_line
 
     def get_log(self): # OK
-        cmd = commands.getstatusoutput('cat /var/log/gufw.log')
-        if cmd[0]:
+        cmd = self._run_cmd(['cat', self.GUFW_LOG])
+        if not cmd:
             return ''
         
-        return cmd[1]
+        return cmd
     
     def get_rules(self):
-        rules = commands.getstatusoutput('LANGUAGE=C ufw status numbered')
-        lines = rules[1].split('\n')
+        rules = self._run_cmd([self.UFW_PATH, 'status', 'numbered'], True)
+        lines = rules.split('\n')
         return_rules = []
         
         for line in lines:
@@ -310,8 +356,8 @@ class Backend():
     
     def get_number_rules(self):
         numb = 0
-        rules = commands.getstatusoutput('LANGUAGE=C ufw status numbered')
-        lines = rules[1].split('\n')
+        rules = self._run_cmd([self.UFW_PATH, 'status', 'numbered'], True)
+        lines = rules.split('\n')
         return_rules = []
         
         for line in lines:
@@ -321,78 +367,96 @@ class Backend():
         return numb
     
     def add_rule(self, insert, policy, direction, iface, routed, logging, proto, from_ip, from_port, to_ip, to_port):
+        # ufw [route] [insert NUM] allow|deny|reject|limit [in|out on INTERFACE] [log|log-all] [proto protocol] [from ADDRESS [port PORT]] [to ADDRESS [port PORT]]
+        cmd_rule = [self.UFW_PATH]
+        
+        # route
+        if routed:
+            cmd_rule.append('route')
+        
         # Insert Number
         if insert:
-            insert = ' '.join(['insert', insert])
+            cmd_rule.extend(['insert', str(int(insert))])
+        
+        # Policy
+        cmd_rule.append(policy)
+        
+        # Direction
+        cmd_rule.append(direction)
+        
         # Interface
         if iface:
-            iface = ' '.join(['on', iface])
-        # Routed
-        pre_route = ''
+            cmd_rule.extend(['on', iface])
+        
+        # Routed on
         if routed:
-            pre_route = 'route'
             if direction == 'in':
-                routed = ' '.join(['out on', routed])
+                cmd_rule.extend(['out', 'on', routed])
             else:
-                routed = ' '.join(['in on', routed])
+                cmd_rule.extend(['in', 'on', routed])
+        
+        # Logging
+        if logging:
+            cmd_rule.append(logging)
+        
+        # Proto
+        if '/tcp' in from_port or '/tcp' in to_port:
+            cmd_rule.extend(['proto', 'tcp'])
+        elif '/udp' in from_port or '/udp' in to_port:
+            cmd_rule.extend(['proto', 'udp'])
+        elif proto:
+            cmd_rule.extend(['proto', proto])
+        
         # From IP
         if not from_ip:
-            from_ip = 'any'
-        from_ip = ' '.join(['from', from_ip])
+            cmd_rule.extend(['from', 'any'])
+        else:
+            cmd_rule.extend(['from', from_ip])
         # From Port
         if from_port:
-            from_port = ' '.join(['port', from_port])
             if '/tcp' in from_port:
                 from_port = from_port.replace('/tcp', '')
-                proto = 'tcp'
             if '/udp' in from_port:
                 from_port = from_port.replace('/udp', '')
-                proto = 'udp'
+            cmd_rule.extend(['port', from_port])
+        
         # To IP
         if not to_ip:
-            to_ip = 'any'
-        to_ip = ' '.join(['to', to_ip])
+            cmd_rule.extend(['to', 'any'])
+        else:
+            cmd_rule.extend(['to', to_ip])
         # To Port
         if to_port:
-            to_port = ' '.join(['port', to_port])
             if '/tcp' in to_port:
                 to_port = to_port.replace('/tcp', '')
-                proto = 'tcp'
             if '/udp' in to_port:
                 to_port = to_port.replace('/udp', '')
-                proto = 'udp'
-        # Protocol
-        if proto:
-            proto = ' '.join(['proto', proto])
+            cmd_rule.extend(['port', to_port])
         
-        # ufw [route] [insert NUM] allow|deny|reject|limit [in|out on INTERFACE] [log|log-all] [proto protocol] [from ADDRESS [port PORT]] [to ADDRESS [port PORT]]
-        # ufw route &insert &policy &direction &iface &routed &logging &proto &fromIP &fromPort &toIP &toPort
-        rule = 'ufw %s %s %s %s %s %s %s %s %s %s %s %s' %  (pre_route, insert, policy, direction, iface, routed, logging, proto, from_ip, from_port, to_ip, to_port)
-        
-        rule = ' '.join(rule.split()) # Condense
-        cmd = commands.getstatusoutput(rule)
+        # Launch
+        cmd = self._run_cmd(cmd_rule, True)
         
         result = []
-        result.append(rule)
-        result.append(cmd[1])
+        result.append(' '.join(cmd_rule))
+        result.append(cmd)
         
         return result # cmd | ufw result
     
     def delete_rule(self, num):
-        delete_rule = 'ufw --force delete &number'.replace('&number', num)
-        cmd = commands.getstatusoutput(delete_rule)
+        delete_rule = [self.UFW_PATH, '--force', 'delete', str(num)]
+        cmd = self._run_cmd(delete_rule)
         
         result = []
-        result.append(delete_rule)
-        result.append(cmd[1])
+        result.append(' '.join(delete_rule))
+        result.append(cmd)
         
         return result # cmd | ufw result
     
     def get_listening_report(self):
         return_report = []
         actual_protocol = 'None'
-        ufw_report = commands.getstatusoutput('LANGUAGE=C ufw show listening')
-        report_lines = ufw_report[1].replace('\n   [', '%')
+        ufw_report = self._run_cmd([self.UFW_PATH, 'show', 'listening'], True)
+        report_lines = ufw_report.replace('\n   [', '%')
         report_lines = report_lines.split('\n')
         
         for descomponent_report in report_lines:
