@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Gufw 15.04.0 - http://gufw.org
+# Gufw 15.10.0 - http://gufw.org
 # Copyright (C) 2008-2015 Marcos Alvarez Costales https://launchpad.net/~costales
 #
 # Gufw is free software; you can redistribute it and/or modify
@@ -90,6 +90,7 @@ class Update:
         self.update_to_port    = self.builder.get_object('update_to_port')
         self.warning           = self.builder.get_object('lbl_preconfig_info')
         self.warning_box       = self.builder.get_object('warningbox')
+        self.update_btn        = self.builder.get_object('btnUpdate')
     
     def _set_initial_values(self):
         iface_num = 0
@@ -125,19 +126,32 @@ class Update:
         self.update_to_ip.set_sensitive(value)
         self.update_from_port.set_sensitive(value)
     
+    def _set_update_btn_control(self):
+        if (':' in self.update_to_port.get_text() or ':' in self.update_from_port.get_text()) and not self.update_protocol.get_active():
+            self.update_btn.set_sensitive(False)
+            self.update_btn.set_tooltip_text(_("Choose a TCP or UDP Protocol with a range of ports"))
+        else:
+            self.update_btn.set_sensitive(True)
+            self.update_btn.set_tooltip_text('')
+    
+    def on_update_protocol_changed(self, widget, data=None):
+        self._set_update_btn_control()
+    
     def on_update_to_port_changed(self, widget, data=None):
-        if ('/' in self.update_to_port.get_text() or
-            '/' in self.update_to_port.get_text()):
+        if '/' in self.update_to_port.get_text():
             self._set_to_port_sensitive(False)
         else:
             self._set_to_port_sensitive(True)
+        
+        self._set_update_btn_control()
     
     def on_update_from_port_changed(self, widget, data=None):
-        if ('/' in self.update_from_port.get_text() or
-            '/' in self.update_from_port.get_text()):
+        if '/' in self.update_from_port.get_text():
             self._set_from_port_sensitive(False)
         else:
             self._set_from_port_sensitive(True)
+        
+        self._set_update_btn_control()
     
     def on_btnUpdateCancel_clicked(self, widget, data=None):
         self.win_update.destroy()
@@ -173,6 +187,14 @@ class Update:
         else:
             self.update_routed.set_sensitive(False)
             self.update_routed.set_tooltip_text(_("You need to set an Interface for forwarding to this another interface"))
+        
+        # Not allow same iface when is routed
+        if self.gufw.fw.get_policy('routed') != 'disabled':
+            self.update_routed.remove_all()
+            self.update_routed.append_text(_("Not Forward"))
+            for ifaceName in self.gufw.fw.get_net_interfaces(self.update_iface.get_active_text()):
+                self.update_routed.append_text(ifaceName)
+            self.update_routed.set_active(0)
     
     def on_btnUpdate_clicked(self, widget, data=None):
         if not self.gufw.validate_rule(self.win_update, self.update_from_ip.get_text(), self.update_from_port.get_text(), self.update_to_ip.get_text(), self.update_to_port.get_text(), '', self.update_routed.get_active_text()):
